var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _class, _temp;

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React from 'react';
import PropTypes from 'prop-types';
import AMapContext from '../AMapContext';
import breakIfNotChildOfAMap from '../utils/breakIfNotChildOfAMap';
import isShallowEqual from '../utils/isShallowEqual';
import createEventCallback from '../utils/createEventCallback';

/**
 * PathNavigator binding.
 * PathNavigator has the same config options as PathNavigator unless highlighted below.
 * For pathNavigator events usage please reference to pathNavigator events paragraph.
 * {@link http://lbs.amap.com/api/javascript-api/reference-amap-ui/mass-data/pathsimplifier#PathNavigator}
 * Besides, it can transform image into pathNavigatorStyle content.
 */
let PathNavigator = (_temp = _class = class PathNavigator extends React.Component {

  /**
   * Parse PathNavigator options.
   * Named properties are event callbacks, other properties are pathNavigator options.
   */

  /**
   * AMap map instance.
   */
  static parsePathNavigatorOptions(props) {
    const {
      pathIndex,
      pathSimplifier,
      PathSimplifierClass,
      onComplete,
      onStart,
      onPause,
      onMove,
      onStop
    } = props,
          pathNavigatorOptions = _objectWithoutProperties(props, ['pathIndex', 'pathSimplifier', 'PathSimplifierClass', 'onComplete', 'onStart', 'onPause', 'onMove', 'onStop']);

    return _extends({}, pathNavigatorOptions, {
      pathNavigatorStyle: _extends({}, pathNavigatorOptions.pathNavigatorStyle, {
        content: (() => {
          if (pathNavigatorOptions.pathNavigatorStyle === void 0) {
            return 'defaultPathNavigator';
          }

          const {
            content
          } = pathNavigatorOptions.pathNavigatorStyle;

          if (typeof content === 'string' && content !== 'defaultPathNavigator' && content !== 'circle' && content !== 'none') {
            return PathSimplifierClass.Render.Canvas.getImageContent(content, () => {
              pathSimplifier.renderLater();
            }, () => {
              throw Error('The image could not be loaded.');
            });
          }

          return content || 'defaultPathNavigator';
        })()
      })
    });
  }

  /**
   * Define event name mapping relations of react binding PathNavigator.
   * Create pathNavigator and bind events.
   * Binding onComplete event on pathNavigator instance.
   */
  constructor(props, context) {
    super(props);

    const {
      onComplete,
      pathSimplifier
    } = props;

    const map = context;

    breakIfNotChildOfAMap('PathNavigator', pathSimplifier, 'PathSimplifier');

    this.pathNavigatorOptions = PathNavigator.parsePathNavigatorOptions(props);

    this.pathNavigator = this.createPathNavigator(this.pathNavigatorOptions);

    this.eventCallbacks = this.parseEvents();

    this.bindEvents(this.pathNavigator, this.eventCallbacks);

    onComplete && onComplete(map, this.pathNavigator, pathSimplifier);
  }

  /**
   * Update this.pathNavigator by calling pathNavigator methods.
   * @param  {Object} nextProps
   * @return {Boolean} - Prevent calling render function
   */
  shouldComponentUpdate(nextProps) {
    const nextPathNavigatorOptions = PathNavigator.parsePathNavigatorOptions(nextProps);

    this.updatePathNavigatorWithApi('setSpeed', this.pathNavigatorOptions.speed, nextPathNavigatorOptions.speed);

    this.setRange(this.pathNavigatorOptions.range, nextPathNavigatorOptions.range);

    this.pathNavigatorOptions = nextPathNavigatorOptions;

    return false;
  }

  /**
   * Remove event listeners.
   * Destroy pathNavigator instance.
   */
  componentWillUnmount() {
    Object.keys(this.eventCallbacks).forEach(key => {
      const eventName = key.substring(2).toLowerCase();
      const handler = this.eventCallbacks[key];

      this.pathNavigator.off(eventName, handler);
    });

    this.pathNavigator.destroy();
    this.pathNavigator = null;
  }

  /**
   * Update pathNavigator range.
   * @param  {Object} currentProp - Current value
   * @param  {Object} nextProp - Next value
   */
  setRange(currentProp, nextProp) {
    if (!isShallowEqual(currentProp, nextProp)) {
      this.pathNavigator.setRange(nextProp[0], nextProp[1]);
    }
  }

  /**
   * Bind all events on pathNavigator instance.
   * Save event listeners.
   * Later to be removed in componentWillUnmount lifecycle.
   * @param  {Object} pathNavigator - PathNavigator instance
   * @param  {Object} eventCallbacks - An object of all event callbacks
   */
  bindEvents(pathNavigator, eventCallbacks) {
    this.AMapEventListeners = [];

    Object.keys(eventCallbacks).forEach(key => {
      const eventName = key.substring(2).toLowerCase();
      const handler = eventCallbacks[key];

      pathNavigator.on(eventName, handler);
    });
  }

  /**
   * Create pathNavigator.
   * @param {Object} pathNavigatorOptions - PathNavigator options
   * @return {PathNavigator}
   */
  createPathNavigator(pathNavigatorOptions) {
    const {
      pathIndex,
      pathSimplifier
    } = this.props;

    const pathNavigator = pathSimplifier.createPathNavigator(pathIndex, pathNavigatorOptions);

    return pathNavigator;
  }

  /**
   * Return an object of all supported event callbacks.
   */
  parseEvents() {
    return {
      onStart: createEventCallback('onStart', this.pathNavigator).bind(this),
      onPause: createEventCallback('onPause', this.pathNavigator).bind(this),
      onMove: createEventCallback('onMove', this.pathNavigator).bind(this),
      onStop: createEventCallback('onStop', this.pathNavigator).bind(this)
    };
  }

  /**
   * Update pathNavigator instance with named api and given value.
   * Won't call api if the given value does not change
   * @param  {string} apiName - PathNavigator instance update method name
   * @param  {Object} currentProp - Current value
   * @param  {Object} nextProp - Next value
   */
  updatePathNavigatorWithApi(apiName, currentProp, nextProp) {
    if (!isShallowEqual(currentProp, nextProp)) {
      this.pathNavigator[apiName](nextProp);
    }
  }

  /**
   * Render nothing.
   */
  render() {
    return null;
  }
}, _class.contextType = AMapContext, _class.propTypes = {
  /**
   * PathSimplifier path index.
   */
  pathIndex: PropTypes.number.isRequired,
  /**
   * AMapUI pathSimplifier instance.
   */
  pathSimplifier: PropTypes.object,
  /**
   * AMapUI pathSimplifier class function.
   */
  PathSimplifierClass: PropTypes.func, // eslint-disable-line react/no-unused-prop-types
  /* eslint-disable react/sort-prop-types,react/no-unused-prop-types */
  /**
   * Event callback.
   *
   * @param {AMap.Map} map                  - AMap.Map instance
   * @param {PathNavigator} PathNavigator   - PathNavigator instance
   * @param {Object} event                  - PathNavigator event parameters
   */
  onComplete: PropTypes.func,
  onStart: PropTypes.func,
  onPause: PropTypes.func,
  onMove: PropTypes.func,
  onStop: PropTypes.func
  /* eslint-enable */
}, _temp);


export default PathNavigator;